"""Tests for the ``check_tinylink_targets`` admin command."""
from django.core import management
from django.test import TestCase

from tinylinks.models import Tinylink
from tinylinks.tests.factories import TinylinkFactory


class CommandTestCase(TestCase):
    """Test class for the ``check_tinylink_targets`` admin command."""
    longMessage = True

    def setUp(self):
        """Prepares the testing environment."""
        # database setup
        self.tinylink_valid = TinylinkFactory(long_url="http://www.google.com")
        self.tinylink_invalid = TinylinkFactory(
            long_url="http://www.a1b2c3d4e5000.com:8888/",
            short_url="cf7GDS",
        )

    def test_command(self):
        """Tests a full run of the custom admin command."""
        management.call_command('check_tinylink_targets')
        # Run twice, because just one link is checked per interval
        management.call_command('check_tinylink_targets')
        self.assertTrue(
            Tinylink.objects.get(pk=self.tinylink_invalid.id).is_broken,
            msg=('Should still be broken.'),
        )
        self.assertFalse(
            Tinylink.objects.get(pk=self.tinylink_valid.id).is_broken,
            msg=('Should be set to not broken.'),
        )



"""
Utilities for creating test objects related to the ``django-tinylinks`` app.

"""
import factory

from django_libs.tests.factories import UserFactory

from tinylinks import models


class TinylinkFactory(factory.DjangoModelFactory):
    FACTORY_FOR = models.Tinylink

    user = factory.SubFactory(UserFactory)
    long_url = "http://www.example.com/thisisalongURL"
    short_url = "vB7f5b"



"""Settings that need to be set in order to run the tests."""
import logging
import os


logging.getLogger("factory").setLevel(logging.WARN)
logging.getLogger("urllib3").setLevel(logging.WARN)

DEBUG = True
USE_TZ = True
SITE_ID = 1

SECRET_KEY = "foobar"

TINYLINK_LENGTH = 5
TINYLINK_CHECK_INTERVAL = 10
TINYLINK_CHECK_PERIOD = 300

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.MD5PasswordHasher',
)

CURRENT_DIR = os.path.dirname(__file__)

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

ROOT_URLCONF = 'tinylinks.tests.urls'

STATIC_URL = '/static/'

STATIC_ROOT = os.path.join(CURRENT_DIR, '../../static/')

STATICFILES_DIRS = (
    os.path.join(CURRENT_DIR, 'test_static'),
)

TEMPLATE_DIRS = (
    os.path.join(CURRENT_DIR, '../templates'),
)

JASMINE_TEST_DIRECTORY = os.path.join(CURRENT_DIR, 'jasmine_tests')

COVERAGE_REPORT_HTML_OUTPUT_DIR = os.path.join(
    CURRENT_DIR, 'coverage')

COVERAGE_MODULE_EXCLUDES = [
    'tests$', 'test_app$', 'settings$', 'urls$', 'locale$',
    'migrations', 'fixtures', 'admin$', 'django_extensions',
]

EXTERNAL_APPS = [
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',
    'django_jasmine',
    'django_libs',
]

INTERNAL_APPS = [
    'django_nose',
    'tinylinks.tests.test_app',
    'tinylinks',
]

INSTALLED_APPS = EXTERNAL_APPS + INTERNAL_APPS

COVERAGE_MODULE_EXCLUDES += EXTERNAL_APPS



"""Tests for the forms of the ``django-tinylinks`` app."""
from django.test import TestCase

from django_libs.tests.factories import UserFactory

from tinylinks.forms import TinylinkForm, TinylinkAdminForm
from tinylinks.models import Tinylink


class TinylinkFormTestCase(TestCase):
    """Test for the ``TinylinkForm`` form class."""
    def test_validates_saves(self):
        # Testing if the new link is saved.
        user = UserFactory()
        data = {'long_url': 'http://www.example.com/FooBar'}

        form = TinylinkForm(data=data, user=user)

        self.assertTrue(form.is_valid(), msg=(
            'If given correct data, the form should be valid.'))
        form.save()
        self.assertEqual(Tinylink.objects.all().count(), 1, msg=(
            'When save is called, there should be one link in the database.'
            ' Got {0}'.format(Tinylink.objects.all().count())))

        # Testing a 'Twin' submit, if the old inputs matches the new ones.
        tinylink = Tinylink.objects.get(pk=1)
        data.update({'short_url': tinylink.short_url})
        form = TinylinkForm(data=data, user=user)
        self.assertFalse(form.is_valid(), msg=(
            'If the short url is already used, the form should be invalid.'))

        # Testing an input with a new short URL. Now, there are two tinylinks
        # with the same long_url.
        data.update({'short_url': 'FooBar01'})
        form = TinylinkForm(data=data, user=user)
        self.assertTrue(form.is_valid(), msg=(
            'If given correct data, the form should be valid.'))
        form.save()
        self.assertEqual(Tinylink.objects.all().count(), 2, msg=(
            'When saving with a new short url, there should be one new'
            ' tinylink in the database. Got {0}'.format(
                Tinylink.objects.all().count())))

        # Testing the input of an old long URL. There's no new submission,
        # only a redirect to the old entity, where this one can be changed.
        form = TinylinkForm(data={'long_url': data['long_url']}, user=user)
        self.assertTrue(form.is_valid(), msg=(
            'If given correct data, the form should be valid.'))
        form.save()
        self.assertEqual(Tinylink.objects.all().count(), 2, msg=(
            'When saving with no short url and the same long url, there is no'
            ' savement. The user is directed to the already existing tinylink.'
            ' Got {0}'.format(Tinylink.objects.all().count())))

        # Testing the changing of a long url
        data = {
            'long_url': 'http://www.example.com/',
            'short_url': tinylink.short_url,
        }
        form = TinylinkForm(instance=tinylink, data=data, user=user,
                            mode="change-long")
        self.assertTrue(form.is_valid(), msg=(
            'If given correct data, the form should be valid.'))
        # If the short_url is owned by another user, throw an error.
        new_user = UserFactory()
        tinylink.user = new_user
        tinylink.save()
        data = {
            'long_url': 'http://www.example.com/',
            'short_url': tinylink.short_url,
        }
        form = TinylinkForm(instance=tinylink, data=data, user=user,
                            mode="change-long")
        self.assertTrue(form.is_valid(), msg=(
            'If the short URL is still unique, the form should be valid.'))
        form.save()
        self.assertEqual(
            Tinylink.objects.filter(short_url=tinylink.short_url).count(), 1)


class TinylinkAdminFormTestCase(TestCase):
    """Test for the ``TinylinkAdminForm`` form class."""
    def test_validates_saves(self):
        # Testing if the new link is saved.
        user = UserFactory()
        data = {
            'long_url': 'http://www.example.com/FooBar',
            'user': user.pk,
        }

        form = TinylinkAdminForm(data=data)

        self.assertTrue(form.is_valid(), msg=(
            'If given correct data, the form should be valid.'))
        form.save()
        self.assertEqual(Tinylink.objects.all().count(), 1, msg=(
            'When save is called, there should be one link in the database.'
            ' Got {0}'.format(Tinylink.objects.all().count())))

        # Testing a 'Twin' submit, if the old inputs matches the new ones.
        tinylink = Tinylink.objects.get(pk=1)
        data.update({'short_url': tinylink.short_url})
        form = TinylinkAdminForm(data=data)
        self.assertFalse(form.is_valid(), msg=(
            'If the short url is already used, the form should be invalid.'))

        # Testing a fake 'Twin' submit, if the old inputs matches the new ones
        # and the object is equal the instance.
        data.update({'short_url': tinylink.short_url})
        form = TinylinkAdminForm(data=data, instance=tinylink)
        self.assertTrue(form.is_valid(), msg=(
            'If the instance is equal the object, the form should be valid.'))



"""URLs for the ``django-tinylinks`` app."""
from django.conf.urls import patterns, url
from django.views.generic import TemplateView

from tinylinks.views import (
    StatisticsView,
    TinylinkCreateView,
    TinylinkDeleteView,
    TinylinkListView,
    TinylinkRedirectView,
    TinylinkUpdateView,
)


urlpatterns = patterns(
    '',
    url(
        r'^$',
        TinylinkListView.as_view(),
        name='tinylink_list'
    ),

    url(
        r'^create/$',
        TinylinkCreateView.as_view(),
        name='tinylink_create'
    ),

    url(
        r'^update/(?P<pk>\d+)/(?P<mode>[a-z-]+)/$',
        TinylinkUpdateView.as_view(),
        name='tinylink_update',
    ),

    url(
        r'^delete/(?P<pk>\d+)/$',
        TinylinkDeleteView.as_view(),
        name='tinylink_delete',
    ),

    url(
        r'^404/$',
        TemplateView.as_view(template_name='tinylinks/notfound.html'),
        name='tinylink_notfound',
    ),

    url(
        r'^statistics/?$',
        StatisticsView.as_view(),
        name='tinylink_statistics',
    ),

    url(
        r'^(?P<short_url>[a-zA-Z0-9-]+)/?$',
        TinylinkRedirectView.as_view(),
        name='tinylink_redirect',
    ),
)



"""Views for the ``django-tinylinks`` application."""
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.http import Http404
from django.utils.decorators import method_decorator
from django.views.generic import (
    CreateView,
    DeleteView,
    ListView,
    RedirectView,
    UpdateView,
)

from tinylinks.forms import TinylinkForm
from tinylinks.models import Tinylink, validate_long_url


class TinylinkViewMixin(object):
    """
    View to handle general functions for Tinylink objects.

    """
    model = Tinylink
    form_class = TinylinkForm

    @method_decorator(permission_required('tinylinks.add_tinylink'))
    def dispatch(self, *args, **kwargs):
        self.mode = kwargs.get('mode')
        return super(TinylinkViewMixin, self).dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(TinylinkViewMixin, self).get_context_data(**kwargs)
        context.update({'mode': self.mode})
        return context

    def get(self, request, *args, **kwargs):
        self.object = None
        if hasattr(self, 'get_object') and kwargs.get('pk'):
            self.object = self.get_object()
            if (not request.user.is_staff
                and (not self.object
                     or not self.object.user == request.user)):
                raise Http404
        return super(TinylinkViewMixin, self).get(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super(TinylinkViewMixin, self).get_form_kwargs()
        kwargs.update({
            'user': self.request.user,
            'mode': self.mode,
        })
        return kwargs

    def get_success_url(self):
        return reverse('tinylink_list')


class TinylinkListView(TinylinkViewMixin, ListView):
    """
    View to list all tinylinks of a user.

    """
    @method_decorator(permission_required('tinylinks.add_tinylink'))
    def dispatch(self, request, *args, **kwargs):
        if request.method == 'POST':
            for key in request.POST:
                if key.startswith('validate'):
                    try:
                        link_id = int(key.replace('validate', ''))
                    except ValueError:
                        raise Http404
                    try:
                        link = Tinylink.objects.get(pk=link_id)
                    except Tinylink.DoesNotExist:
                        raise Http404
                    validate_long_url(link)
        return super(TinylinkListView, self).dispatch(request, *args, **kwargs)

    def get_queryset(self):
        if self.request.user.is_staff:
            return Tinylink.objects.all()
        return self.request.user.tinylinks.all()


class TinylinkCreateView(TinylinkViewMixin, CreateView):
    """
    View to generate a Tinylink instance including a shortened URL.

    """
    def get_success_url(self):
        return reverse('tinylink_update', kwargs={'pk': self.object.id,
                                                  'mode': 'change-short'})


class TinylinkUpdateView(TinylinkViewMixin, UpdateView):
    """
    View to update a Tinylink instance.

    """
    pass


class TinylinkDeleteView(TinylinkViewMixin, DeleteView):
    """
    View to delete a certain tinylink.

    """
    pass


class TinylinkRedirectView(RedirectView):
    """
    View to validate a short URL and redirect to its location.

    """
    def dispatch(self, *args, **kwargs):
        if kwargs.get('short_url'):
            try:
                tinylink = Tinylink.objects.get(short_url=kwargs.get(
                    'short_url'))
            except Tinylink.DoesNotExist:
                tinylink = None
                self.url = reverse('tinylink_notfound')
            if tinylink:
                # set the redirect long URL
                self.url = tinylink.long_url
                tinylink.amount_of_views += 1
                tinylink.save()
        return super(TinylinkRedirectView, self).dispatch(*args, **kwargs)

    def get_redirect_url(self, **kwargs):
        """
        We have to override this method.

        The original method tries to do `self.url % kwargs` which will fail
        when the URL has `%` characters.

        """
        if self.url:
            url = self.url
            args = self.request.META.get('QUERY_STRING', '')
            if args and self.query_string:
                url = "%s?%s" % (url, args)
            return url
        else:
            return None


class StatisticsView(ListView):
    """
    View to list all tinylinks including their statistics.

    """
    model = Tinylink
    template_name = "tinylinks/statistics.html"

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            raise Http404
        return super(StatisticsView, self).dispatch(request, *args, **kwargs)



"""
This ``urls.py`` is only used when running the tests via ``runtests.py``.
As you know, every app must be hooked into yout main ``urls.py`` so that
you can actually reach the app's views (provided it has any views, of course).

"""
from django.conf.urls import include, patterns, url
from django.contrib import admin

from tinylinks.tests.views import TestFailedRedirectView, TestRedirectView


admin.autodiscover()


urlpatterns = patterns(
    '',
    url(r'^redirect-test/', TestRedirectView.as_view()),
    url(r'^redirect-fail/', TestFailedRedirectView.as_view()),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^', include('tinylinks.urls')),
)



"""Admin sites for the ``django-tinylinks`` app."""
from django.contrib import admin
from django.template.defaultfilters import truncatechars
from django.utils.translation import ugettext_lazy as _

from tinylinks.forms import TinylinkAdminForm
from tinylinks.models import Tinylink


class TinylinkAdmin(admin.ModelAdmin):
    list_display = ('short_url', 'url_truncated', 'amount_of_views', 'user',
                    'last_checked', 'status', 'validation_error')
    search_fields = ['short_url', 'long_url']
    form = TinylinkAdminForm

    def url_truncated(self, obj):
        return truncatechars(obj.long_url, 60)
    url_truncated.short_description = _('Long URL')

    def status(self, obj):
        if not obj.is_broken:
            return _('OK')
        return _('Link broken')
    status.short_description = _('Status')


admin.site.register(Tinylink, TinylinkAdmin)



"""
Custom admin command to check all tinylink target URLs.

It should check in a certain interval during a certain period defined in the
settings by TINYLINK_CHECK_INTERVAL and TINYLINK_CHECK_PERIOD.
After one period, all URLs should be checked for their availability.

"""
from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils import timezone

from tinylinks.models import Tinylink, validate_long_url


class Command(BaseCommand):
    """Class for the check_tinylink_targets admin command."""
    def handle(self, *args, **options):
        """Handles the check_tinylink_targets admin command."""
        interval = settings.TINYLINK_CHECK_INTERVAL
        period = settings.TINYLINK_CHECK_PERIOD
        url_amount = Tinylink.objects.all().count()
        check_amount = (url_amount / (period / interval)) or 1
        for link in Tinylink.objects.order_by('last_checked')[:check_amount]:
            validate_long_url(link)
        print('[' + timezone.now().strftime('%d.%m.%Y - %H:%M') +
              '] Checked ' + str(check_amount) + ' of ' + str(url_amount) +
              ' total URLs.')



"""Forms for the ``django-tinylinks`` app."""
import random

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from tinylinks.models import Tinylink, validate_long_url


class TinylinkForm(forms.ModelForm):
    """
    Creates and validates long and short URL version.

    """
    def __init__(self, user=None, mode='change-short', *args, **kwargs):
        """
        The Regex field validates the URL input. Allowed are only slugified
        inputs.

        Examples:
        "D834n-qNx2q-jn" <- valid
        "D834n qNx2q jn" <- invalid
        "D834n_qNx2q/jn" <- invalid
        """
        super(TinylinkForm, self).__init__(*args, **kwargs)
        if mode == 'change-long':
            long_help_text = _("You can now change your long URL.")
        else:
            long_help_text = _("The long URL isn't editable at the moment.")
        self.fields['long_url'] = forms.URLField(
            label=self.instance._meta.get_field_by_name(
                'long_url')[0].verbose_name,
            help_text=long_help_text,
        )
        if not self.instance.pk:
            # Hide the short URL field to auto-generate a new instance.
            self.fields['short_url'].widget = forms.HiddenInput()
            self.fields['short_url'].required = False
        else:
            # Dependent on the user mode, one URL field should not be editable.
            if mode == 'change-long':
                self.fields['short_url'].widget.attrs['readonly'] = True
                self.fields['short_url'].help_text = _(
                    "The short URL isn't editable at the moment.")
            else:
                self.fields['long_url'].widget.attrs['readonly'] = True
                self.fields['short_url'] = forms.RegexField(
                    regex=r'^[a-z0-9]+$',
                    error_message=(_("Please use only small letters and"
                                     " digits.")),
                    help_text=_("You can add a more readable short URL."),
                    label=self.instance._meta.get_field_by_name(
                        'short_url')[0].verbose_name,
                )
        self.user = user

    def clean(self):
        self.cleaned_data = super(TinylinkForm, self).clean()
        # If short URL is occupied throw out an error, or fail silent.
        try:
            twin = Tinylink.objects.get(short_url=self.cleaned_data.get(
                'short_url'))
            if not self.instance == twin:
                self._errors['short_url'] = forms.util.ErrorList([_(
                    'This short url already exists. Please try another one.')])
            return self.cleaned_data
        except Tinylink.DoesNotExist:
            pass
        # Brothers are entities with the same long URL
        brothers = Tinylink.objects.filter(long_url=self.cleaned_data.get(
            'long_url'), user=self.user)
        input_url = self.cleaned_data.get('short_url')

        # Only handle with older brothers, if there's no new short URL value
        if brothers and not input_url:
            # This can only happen, if a user tries to auto-generate a
            # short URL with an existing tinylink. She will receive the
            # prefilled form with the link's old values.
            self.instance = brothers[0]
            self.cleaned_data.update(
                {'short_url': self.instance.short_url})
        else:
            slug = ''
            if input_url:
                # User can customize their URLs
                slug = input_url
            # This keeps the unique validation of the short URLs alive.
            if not Tinylink.objects.filter(short_url=input_url):
                while not slug or Tinylink.objects.filter(short_url=slug):
                    slug = ''.join(
                        random.choice('abcdefghijkmnpqrstuvwxyz123456789')
                        for x in range(
                            getattr(settings, 'TINYLINK_LENGTH', 6)))
                self.cleaned_data.update({'short_url': slug})
        return self.cleaned_data

    def save(self, *args, **kwargs):
        if not self.instance.pk:
            self.instance.user = self.user
        self.instance = super(TinylinkForm, self).save(*args, **kwargs)
        return validate_long_url(self.instance)

    class Meta:
        model = Tinylink
        fields = ('long_url', 'short_url')


class TinylinkAdminForm(forms.ModelForm):
    """
    Creates and updates long and short URL versions in the Django Admin.

    """
    def __init__(self, *args, **kwargs):
        """The Regex field validates the URL input."""
        super(TinylinkAdminForm, self).__init__(*args, **kwargs)
        if not self.instance.pk:
            # Hide the short URL field to auto-generate a new instance.
            self.fields['short_url'].widget = forms.HiddenInput()
            self.fields['short_url'].required = False
        else:
            self.fields['short_url'] = forms.RegexField(
                regex=r'^[a-z0-9]+$',
                error_message=(_("Please use only small letters and digits.")),
                help_text=_("You can add a more readable short URL."),
                label=self.instance._meta.get_field_by_name(
                    'short_url')[0].verbose_name,
            )

    def clean(self):
        self.cleaned_data = super(TinylinkAdminForm, self).clean()
        # If short URL is occupied throw out an error, or fail silent.
        try:
            twin = Tinylink.objects.get(
                short_url=self.cleaned_data.get('short_url'),
            )
        except Tinylink.DoesNotExist:
            slug = self.cleaned_data.get('short_url')
            while not slug or Tinylink.objects.filter(short_url=slug):
                slug = ''.join(
                    random.choice('abcdefghijkmnpqrstuvwxyz123456789')
                    for x in range(getattr(settings, 'TINYLINK_LENGTH', 6)))
            self.cleaned_data.update({'short_url': slug})
        else:
            if twin != self.instance:
                self._errors['short_url'] = forms.util.ErrorList([_(
                    'This short url already exists. Please try another one.')])
        return self.cleaned_data

    class Meta:
        model = Tinylink
        fields = ('user', 'long_url', 'short_url')



"""Models for the ``django-tinylinks`` app."""
import urllib2
from cookielib import CookieJar
from socket import gaierror

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

from urllib3 import PoolManager
from urllib3.exceptions import HTTPError, MaxRetryError, TimeoutError


def get_url_response(pool, link, url):
    """
    Function to open and check an URL. In case of failure it sets the relevant
    validation error.

    """
    response = False
    link.is_broken = True
    link.redirect_location = ''
    # Try to encode e.g. chinese letters
    try:
        url = url.encode('utf-8')
    except UnicodeEncodeError:
        link.validation_error = _('Unicode error. Check URL characters.')
        return False
    try:
        response = pool.urlopen('GET', url, retries=2, timeout=8.0)
    except TimeoutError:
        link.validation_error = _('Timeout after 8 seconds.')
    except MaxRetryError:
        link.validation_error = _('Failed after retrying twice.')
    except (HTTPError, gaierror):
        link.validation_error = _('Not found.')
    return response


def validate_long_url(link):
    """
    Function to validate a URL. The validator uses urllib3 to test the URL's
    availability.

    """
    http = PoolManager()
    response = get_url_response(http, link, link.long_url)
    if response and response.status == 200:
        link.is_broken = False
    elif response and response.status == 302:
        # If link is redirected, validate the redirect location.
        if link.long_url.endswith('.pdf'):
            # Non-save pdf exception, to avoid relative path redirects
            link.is_broken = False
        else:
            redirect_location = response.get_redirect_location()
            redirect = get_url_response(http, link, redirect_location)
            link.redirect_location = redirect_location
            if redirect.status == 200:
                link.is_broken = False
            elif redirect.status == 302:
                # Seems like an infinite loop. Maybe the server is looking for
                # a cookie?
                cj = CookieJar()
                opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
                request = urllib2.Request(response.get_redirect_location())
                response = opener.open(request)
                if response.code == 200:
                    link.is_broken = False
    elif response and response.status == 502:
        # Sometimes urllib3 repond with a 502er. Those pages might respond with
        # a 200er in the Browser, so re-check with urllib2
        try:
            response = urllib2.urlopen(link.long_url, timeout=8.0)
        except urllib2.HTTPError:
            link.validation_error = _("URL not accessible.")
        else:
            link.is_broken = False
    else:
        link.validation_error = _("URL not accessible.")
    link.last_checked = timezone.now()
    link.save()
    return link


class Tinylink(models.Model):
    """
    Model to 'translate' long URLs into small ones.

    :user: The author of the tinylink.
    :long_url: Long URL version.
    :short_url: Shortened URL.
    :is_broken: Set if the given long URL couldn't be validated.
    :validation_error: Description of the occurred error.
    :last_checked: Datetime of the last validation process.
    :amount_of_views: Field to count the redirect views.
    :redirect_location: Redirect location if the long_url is redirected.

    """
    user = models.ForeignKey(
        'auth.User',
        verbose_name=_('Author'),
        related_name="tinylinks",
    )

    long_url = models.CharField(
        max_length=2500,
        verbose_name=_('Long URL'),
    )

    short_url = models.CharField(
        max_length=32,
        verbose_name=_('Short URL'),
        unique=True,
    )

    is_broken = models.BooleanField(
        default=False,
        verbose_name=_('Status'),
    )

    validation_error = models.CharField(
        max_length=100,
        verbose_name=_('Validation Error'),
        default='',
    )

    last_checked = models.DateTimeField(
        default=timezone.now(),
        verbose_name=_('Last validation'),
    )

    amount_of_views = models.PositiveIntegerField(
        default=0,
        verbose_name=_('Amount of views'),
    )

    redirect_location = models.CharField(
        max_length=2500,
        verbose_name=_('Redirect location'),
        default='',
    )

    def __unicode__(self):
        return self.short_url

    class Meta:
        ordering = ['-id']

    def can_be_validated(self):
        """
        URL can only be validated if the last validation was at least 1
        hour ago

        """
        if self.last_checked < timezone.now() - timezone.timedelta(minutes=60):
            return True
        return False



"""Tests for the models of the ``django-tinylinks`` app."""
from django.test import TestCase, LiveServerTestCase
from django.utils import timezone

from tinylinks.models import validate_long_url, Tinylink
from tinylinks.tests.factories import TinylinkFactory


class TinylinkTestCase(TestCase, LiveServerTestCase):
    """Tests for the ``Tinylink`` model class."""
    def test_model(self):
        obj = TinylinkFactory()
        self.assertTrue(obj.pk)

        link = TinylinkFactory(long_url="http://www.google.com/",
                               short_url="v4bG4S")
        validate_long_url(link)
        self.assertEqual(Tinylink.objects.get(pk=link.pk).validation_error, "")
        link.long_url = "http://www.a1b2c3d4e5000.com:8888/"
        link.save()
        validate_long_url(link)
        self.assertEqual(Tinylink.objects.get(pk=link.pk).validation_error,
                         "URL not accessible.")
        link.long_url = "%s/redirect-fail/" % self.live_server_url
        link.save()
        validate_long_url(link)
        self.assertTrue(Tinylink.objects.get(pk=link.pk).is_broken)
        link.long_url = "%s/redirect-test/" % self.live_server_url
        link.save()
        validate_long_url(link)
        self.assertTrue(Tinylink.objects.get(pk=link.pk).is_broken)

        link.last_checked = timezone.now() - timezone.timedelta(minutes=61)
        self.assertTrue(link.can_be_validated())



"""Test views for the ``django-tinylinks`` application."""
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect
from django.views.generic import View


class TestFailedRedirectView(View):
    def dispatch(self, *args, **kwargs):
        raise Http404


class TestRedirectView(View):
    def dispatch(self, *args, **kwargs):
        return HttpResponseRedirect(reverse('tinylink_create'))
